package de.opalpiraten.scenepainter.gui;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.prefs.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * An Adapter for displaying the Scene on the background of the workspace.
 * This class will also make sure, sceneelements use the current project
 * as loader.
 */
public class Viewer extends JPanel implements WorkspaceListener, 
SceneListener, ActionListener, ComponentListener {

  /**
   * The Scene showing
   */
  private Scene scene;
  
  /**
   * For scrolling the scene around
   */
  private JScrollPane scroll;
  
  /**
   * For plugin into the configmenu
   */
  private JMenuItem item = new JMenuItem(Dict.get("gui.viewer.item"));
  
  /**
   * Sole constructor
   */
  public Viewer() {
    item.addActionListener(this);
    item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2,0));
    Menus.addToConfig(item);
    // This is a hacky way to make it look like as if this was a window
    // just without a title: Set the background to the color of an inactive
    // JInternalframe and make the scroll 5 px smaller on each side.
    // Of course, this only looks good and consistend with the default
    // theme.
    setBackground(Color.decode("#7A8A99"));
  }
  
  public void replaceProject(Project p) {
    // Make sure, the sceneelements use the current project as a loader.
    if (scene==null) return;
    SceneElement se[] = scene.getElements();
    for(int i=0;i<se.length;i++) {
      se[i].setProject(p);
    }
    repaint();
  }
  
  public void replaceScene(Scene s) {
    if (scene==s) return;
    scene=s;
    if(scroll!=null) remove(scroll);
    if (scene!=null) scene.removeSceneListener(this);
    scene.addSceneListener(this);
    scroll = new JScrollPane(s);
    scroll.setBorder(null);
    add(scroll);
    validate();
  }
  
  public void doLayout() {
    Rectangle bounds = getBounds();
    scroll.setBounds(5,5,bounds.width-10,bounds.height-10);
    repaint();
  }
  
  public void sceneChanged(Scene s) {
    scroll.doLayout();
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    if (src==item) {
      JInternalFrame jif = new JInternalFrame(Dict.get("gui.viewer.title"),true,true,true);
      jif.setBounds(getBounds());
      JTextArea txt = new JTextArea(Dict.get("gui.viewer.rebound"));
      txt.setEditable(false);
      JPanel content= new JPanel();
      content.setLayout(new GridBagLayout());
      content.add(txt);
      txt.setBackground(jif.getBackground());
      jif.setContentPane(content);
      jif.addComponentListener(this);
      getParent().add(jif);
      jif.show();
    }
  }
  
  public void componentHidden(ComponentEvent e) {}
  public void componentShown(ComponentEvent e) {}

  public void componentMoved(ComponentEvent e) {
    setBounds(((Component)e.getSource()).getBounds());
    validate();
  }
  
  public void componentResized(ComponentEvent e) {
    setBounds(((Component)e.getSource()).getBounds());
    validate();
  }

}
