package de.opalpiraten.scenepainter.gui.render;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * A Grid is a visual aid painted on the Scene, dividing it into cells.
 */
public class Grid {

  /**
   * The rectangular size of one cell
   */
  private Dimension cellsize;
  
  /**
   * The color to draw the lines with
   */
  private Color linecolor = Color.gray.brighter();
  
  /**
   * Construct a new grid. Layoutinformation (cell size, line color, etc.)
   * are automatically fetched from the runtime config and can only be changed
   * there
   */
  public Grid() {
    cellsize = new Dimension(10,10);
  }
  
  /**
   * Set the cellsize. No matter how the grid look visually, the cells are
   * always rectangular.
   */
  public void setCellsize(Dimension d) {
    cellsize =d;
  }
  
  /**
   * Query the cellsize
   */
  public Dimension getCellsize() {
    return cellsize;
  }
  
  /**
   * Set the Color for drawing the Grid
   */
  public void setLinecolor(Color c) {
    if (c==null) return;
    linecolor=c;
  }
  
  /**
   * Query the linecolor
   */
  public Color getLinecolor() {
    return linecolor;
  }
  
  /**
   * Called by the Scene to draw the grid.
   */
  public void paint(Graphics2D gfx, int width, int height) {
    gfx.setColor(linecolor);
    for(int x = cellsize.width;x<width;x=x+cellsize.width) {
      gfx.drawLine(x,0,x,height);
    }
    for(int y=cellsize.height;y<height;y=y+cellsize.height) {
      gfx.drawLine(0,y,width,y);
    }
  }
  
  /**
   * Query the location of the origin of a cell, a specific Point resides
   * in. This method is intended to be used for "snap to grid" placement of 
   * SceneElements. The idea is to submit the coordinates of the mousepointer 
   * and get the point on the grid, where the SceneElement should be placed.
   * @param p an arbitary coordinate on the grid.
   * @return the coordinate of the origin of the cell, the given point
   * is in.
   */
  public Point getCellOrigin(Point coord) {
    return new Point((coord.x/cellsize.width)*cellsize.width,(coord.y/cellsize.height)*cellsize.height);
  }
    
}
